 /*
  * sigact.c
  * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
  * 
  * simple-netaid is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  * 
  * simple-netaid is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * See the COPYING file.
  */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <inttypes.h>
#include <errno.h>
#include <assert.h>
#include <pstat/libpstat.h>

#include <cdk_test.h>

#include "sigact.h"
#include <globals.h>

#define MAX_SIZE 1024
#define MAX_PTH  128

// Usage: strncpy_t(str1, sizeof(str1), str2, strlen(str2));
// Copy string "in" with at most "insz" chars to buffer "out", which
// is "outsz" bytes long. The output is always 0-terminated. Unlike
// strncpy(), strncpy_t() does not zero fill remaining space in the
// output buffer:
static
char* strncpy_t(char* out, size_t outsz, const char* in, size_t insz)
{
   assert(outsz > 0);
   while(--outsz > 0 && insz > 0 && *in) { *out++ = *in++; insz--; }
   *out = 0;
   return out;
}

static
void terminate_nl_monitor()
{
	DIR *dirp = NULL;
	struct dirent *dir = NULL;
	
	dirp=opendir("/proc");
	if(!dirp) {
		printf("\n\tERROR: unable to open /proc for read\n\n");
		exit(EXIT_FAILURE);
	}
	
	while((dir=readdir(dirp)) != NULL ) {

        int rc = 0;
	    struct stat sb;
        char *proc_cmd = NULL;
        bool is_uint_ok = true;
        char pth[MAX_PTH] = {0};
	    
	    for(unsigned i=0; i<strlen(dir->d_name); i++) {
		   if(!isdigit(dir->d_name[i])) is_uint_ok = false;
	    }
        
        if(!is_uint_ok) continue;
        
        strncpy_t(pth, sizeof(pth), "/proc/", strlen("/proc/"));
        strcat(pth, dir->d_name);
        if (stat(pth, &sb) == -1) {
           perror("stat");
           exit(EXIT_FAILURE);
        }
        
        if(!lstat(dir->d_name, &sb)) continue;
 
        /* Only consider directories */
        if(!S_ISDIR(sb.st_mode)) continue;

		rc = pstat_get_binary_path_from_string(&proc_cmd, MAX_PTH, dir->d_name);
		if(rc) continue;
		
		if(!strcmp(rindex(proc_cmd, '/') + 1, "nl_monitor")) {
              if( (kill(atoi(dir->d_name), SIGTERM)) != 0 ) kill(atoi(dir->d_name), SIGKILL);
        }
        
	    free(proc_cmd);
	}
	
    closedir(dirp);
}    

static
void hdl(int signum, siginfo_t *info, void *extra)
{
   switch(signum)
   {
      case SIGALRM:
         break;
      case SIGCHLD:
         break;
      case SIGHUP:
         break;
      case SIGUSR1:
         //signal(SIGUSR1, SIG_IGN);
         ui_update();
         sigaction_init();
         break;
      case SIGUSR2:
         break;
      case SIGTERM:
      case SIGINT:
      case SIGQUIT:
         terminate_nl_monitor();
         /* Make sure to cleanup if user sends a keyboard interrupt */
         ui_delete();
         break;
   }
}

void sigaction_init()
{
   struct sigaction sa;
   memset (&sa, 0, sizeof(sa));
      
   /*---------------- initialize signal handling ------------ */
   sa.sa_sigaction = &hdl;
   sa.sa_flags = SA_RESETHAND | SA_RESTART | SA_NODEFER | SA_SIGINFO;
   sigemptyset(&sa.sa_mask);
   sigaction(SIGALRM, &sa, 0);
   sigaction(SIGCHLD, &sa, 0);
   sigaction(SIGHUP,  &sa, 0);
   sigaction(SIGUSR1, &sa, 0);
   sigaction(SIGUSR2, &sa, 0);
   sigaction(SIGTERM, &sa, 0);
   sigaction(SIGINT,  &sa, 0);
   sigaction(SIGQUIT, &sa, 0);
}
